package controllers

import (
	"errors"
	"net/http"
	"spaceofcheng/services/service"
	"spaceofcheng/services/utils"
	"strings"

	"github.com/astaxie/beego"
)

// Authenticate 权限验证
func (c *BaseController) authenticate() {
	// 默认 service
	serv := service.NewSysServ(c.Context)

	// 客户端类型
	clientType := utils.GetClientType(c.Ctx.Request)

	switch clientType {
	case utils.ClientAdmin:
		c.authPCAdmin(serv)
	case utils.ClientWechat:
		c.authWechat(serv)
	default:
		c.ResponseError(
			errors.New("暂不支持的 API 场景"),
			http.StatusBadRequest,
		)
	}
}

// authPCAdmin PC管理端
func (c *BaseController) authPCAdmin(serv *service.SysServ) {
	if !c.needAuth() {
		return
	}

	// 用户ID
	userID := ""
	userIDRaw := c.GetSession(SNUserID)
	if userIDRaw != nil {
		userID = userIDRaw.(string)
	}

	if userID == "" {
		c.ResponseError(
			errors.New("用户未登录"),
			http.StatusUnauthorized,
		)
	}

	if err := serv.SetUserProfile(userID); err != nil {
		// utils.LogError(err.Error())

		c.ResponseError(
			utils.LogError(err.Error()),
			http.StatusInternalServerError,
		)
	}

	// 设置 Session
	c.SetSession(SNUserID, userID)
}

// authWechat 微信端
func (c *BaseController) authWechat(serv *service.SysServ) {
	custID := ""
	custIDRaw := c.GetSession(SNCustID)
	if custIDRaw != nil {
		custID = custIDRaw.(string)
	}

	// 用户微信信息
	var wxDetail map[string]interface{}
	detailRaw := c.GetSession("wechat_user")
	if detailRaw != nil {
		wxDetail = detailRaw.(map[string]interface{})
	}

	if custID == "" && wxDetail == nil {
		wxDetail = c.getWechatDetail()
		c.SetSession("wechat_user", wxDetail)
	}

	c.Context.Set("wxInfo", wxDetail)

	// 普通客户映射检查
	userMap, err := serv.CheckWechatUserMapping(wxDetail)
	if err != nil {
		c.ResponseError(
			utils.LogError("获取人员映射账户失败: "+err.Error()),
			http.StatusInternalServerError,
		)
	}

	if !c.needAuth() {
		return
	}

	// 未登录或者绑定, 返回 401
	if custID == "" && userMap.UserId == "" {
		wxConf, ok := c.Configer[WeChatConf]
		if ok {
			c.ResponseData(
				map[string]interface{}{
					"appid": wxConf.String("appid"),
				},
				errors.New("用户未登录"),
				http.StatusUnauthorized,
			)
		} else {
			c.ResponseError(
				errors.New("用户未登录"),
				http.StatusUnauthorized,
			)
		}
	}

	if custID == "" {
		custID = userMap.UserId
	}

	if err := serv.SetCustomer(custID); err != nil {
		utils.LogError(err.Error())

		c.ResponseError(
			errors.New("内部错误, 请重试"),
			http.StatusInternalServerError,
		)
	}

	// 设置 Session
	c.SetSession(SNCustID, custID)
}

func (c *BaseController) needAuth() bool {
	route := c.Ctx.Input.URL()
	apiPrefix := beego.AppConfig.String("api::prefix")
	guestAPI := beego.AppConfig.String("api::guest")

	if strings.Index(route, apiPrefix+guestAPI) > -1 {
		return false
	}

	return true
}

// getWechatDetail 获取微信个人信息详情
func (c *BaseController) getWechatDetail() map[string]interface{} {
	// 微信 code
	code := c.GetString("code")
	if code == "" {
		c.ResponseError(
			errors.New("没有接收到微信 code"),
			http.StatusBadRequest,
		)
	}

	usr, err := utils.GetWxUserInfo(code)
	if err != nil {
		c.ResponseError(
			utils.LogError("获取微信个人信息失败: "+err.Error()),
			http.StatusInternalServerError,
		)
	}

	return usr
}
